@@ -16,6 +16,7 @@
 #                       1024/client-cert.pem
 #                       server-ecc-comp.pem
 #                       client-ca.pem
+#                       test/digsigku.pem
 # updates the following crls:
 #                       crl/cliCrl.pem
 #                       crl/crl.pem
@@ -53,6 +54,24 @@ function run_renewcerts(){
 
     openssl x509 -in client-cert.pem -text > tmp.pem
     mv tmp.pem client-cert.pem
+
+
+    ############################################################
+    #### update the self-signed (3072-bit) client-cert.pem #####
+    ############################################################
+    echo "Updating 3072-bit client-cert.pem"
+    echo ""
+    #pipe the following arguments to openssl req...
+    echo -e "US\nMontana\nBozeman\nwolfSSL_3072\nProgramming-3072\nwww.wolfssl.com\ninfo@wolfssl.com\n.\n.\n" | openssl req -new -newkey rsa:3072 -keyout client-key-3072.pem -nodes -out client-cert-3072.csr
+
+
+    openssl x509 -req -in client-cert-3072.csr -days 1000 -extfile wolfssl.cnf -extensions wolfssl_opts -signkey client-key-3072.pem -out client-cert-3072.pem
+    rm client-cert-3072.csr
+
+    openssl x509 -in client-cert-3072.pem -text > tmp.pem
+    mv tmp.pem client-cert-3072.pem
+
+
     ############################################################
     #### update the self-signed (1024-bit) client-cert.pem #####
     ############################################################
@@ -128,6 +147,23 @@ function run_renewcerts(){
     cat ca_tmp.pem >> server-revoked-cert.pem
     rm ca_tmp.pem
     ###########################################################
+    ########## update and sign server-duplicate-policy.pem ####
+    ###########################################################
+    echo "Updating server-duplicate-policy.pem"
+    echo ""
+    #pipe the following arguments to openssl req...
+    echo -e "US\nMontana\nBozeman\nwolfSSL\ntesting duplicate policy\nwww.wolfssl.com\ninfo@wolfssl.com\n.\n.\n" | openssl req -new -key server-key.pem -nodes > ./test/server-duplicate-policy-req.pem
+
+    openssl x509 -req -in ./test/server-duplicate-policy-req.pem -extfile wolfssl.cnf -extensions policy_test -days 1000 -CA ca-cert.pem -CAkey ca-key.pem -set_serial 02 > ./test/server-duplicate-policy.pem
+
+    rm ./test/server-duplicate-policy-req.pem
+
+    openssl x509 -in ca-cert.pem -text > ca_tmp.pem
+    openssl x509 -in ./test/server-duplicate-policy.pem -text > srv_tmp.pem
+    mv srv_tmp.pem ./test/server-duplicate-policy.pem
+    cat ca_tmp.pem >> ./test/server-duplicate-policy.pem
+    rm ca_tmp.pem
+    ###########################################################
     #### update and sign (1024-bit) server-cert.pem ###########
     ###########################################################
     echo "Updating 1024-bit server-cert.pem"
@@ -208,6 +244,22 @@ function run_renewcerts(){
     echo ""
     cat client-cert.pem client-ecc-cert.pem > client-ca.pem
 
+    ############################################################
+    ###### update the self-signed test/digsigku.pem   ##########
+    ############################################################
+    echo "Updating test/digsigku.pem"
+    echo ""
+    #pipe the following arguments to openssl req...
+    echo -e "US\nWashington\nSeattle\nFoofarah\nArglebargle\nfoobarbaz\ninfo@worlss.com\n.\n.\n" | openssl req -new -key ecc-key.pem -nodes -sha1 -out digsigku.csr
+
+
+    openssl x509 -req -in digsigku.csr -days 1000 -extfile wolfssl.cnf -extensions digsigku -signkey ecc-key.pem -sha1 -set_serial 16393466893990650224 -out digsigku.pem
+    rm digsigku.csr
+
+    openssl x509 -in digsigku.pem -text > tmp.pem
+    mv tmp.pem digsigku.pem
+    mv digsigku.pem test/digsigku.pem
+
     ############################################################
     ########## make .der files from .pem files #################
     ############################################################
