@@ -1,6 +1,6 @@
 /* evp.c
  *
- * Copyright (C) 2006-2016 wolfSSL Inc.
+ * Copyright (C) 2006-2017 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -19,8 +19,53 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
+#if !defined(WOLFSSL_EVP_INCLUDED)
+    #warning evp.c does not need to be compiled seperatly from ssl.c
+#else
+
 static unsigned int cipherType(const WOLFSSL_EVP_CIPHER *cipher);
 
+#ifdef WOLFSSL_SIGNAL
+WOLFSSL_API int  wolfSSL_EVP_EncryptInit(WOLFSSL_EVP_CIPHER_CTX* ctx,
+                                        const WOLFSSL_EVP_CIPHER* type,
+                                        const unsigned char* key,
+                                        const unsigned char* iv)
+{
+    return wolfSSL_EVP_CipherInit(ctx, type, (byte*)key, (byte*)iv, 1);
+}
+
+WOLFSSL_API int  wolfSSL_EVP_EncryptInit_ex(WOLFSSL_EVP_CIPHER_CTX* ctx,
+                                        const WOLFSSL_EVP_CIPHER* type,
+                                        WOLFSSL_ENGINE *impl,
+                                        const unsigned char* key,
+                                        const unsigned char* iv)
+{
+    (void) impl;
+    return wolfSSL_EVP_CipherInit(ctx, type, (byte*)key, (byte*)iv, 1);
+}
+
+WOLFSSL_API int  wolfSSL_EVP_DecryptInit(WOLFSSL_EVP_CIPHER_CTX* ctx,
+                                        const WOLFSSL_EVP_CIPHER* type,
+                                        const unsigned char* key,
+                                        const unsigned char* iv)
+{
+    WOLFSSL_ENTER("wolfSSL_EVP_CipherInit");
+    return wolfSSL_EVP_CipherInit(ctx, type, (byte*)key, (byte*)iv, 0);
+}
+
+WOLFSSL_API int  wolfSSL_EVP_DecryptInit_ex(WOLFSSL_EVP_CIPHER_CTX* ctx,
+                                        const WOLFSSL_EVP_CIPHER* type,
+                                        WOLFSSL_ENGINE *impl,
+                                        const unsigned char* key,
+                                        const unsigned char* iv)
+{
+    (void) impl;
+    WOLFSSL_ENTER("wolfSSL_EVP_DecryptInit");
+    return wolfSSL_EVP_CipherInit(ctx, type, (byte*)key, (byte*)iv, 0);
+}
+
+#else /* WOLFSSL_SIGNAL */
+
 WOLFSSL_API int  wolfSSL_EVP_EncryptInit(WOLFSSL_EVP_CIPHER_CTX* ctx,
                                         const WOLFSSL_EVP_CIPHER* type,
                                         unsigned char* key, unsigned char* iv)
@@ -55,12 +100,14 @@ WOLFSSL_API int  wolfSSL_EVP_DecryptInit_ex(WOLFSSL_EVP_CIPHER_CTX* ctx,
     return wolfSSL_EVP_CipherInit(ctx, type, key, iv, 0);
 }
 
+#endif /* WOLFSSL_SIGNAL */
+
 WOLFSSL_API WOLFSSL_EVP_CIPHER_CTX *wolfSSL_EVP_CIPHER_CTX_new(void)
 {
 	WOLFSSL_EVP_CIPHER_CTX *ctx = (WOLFSSL_EVP_CIPHER_CTX*)XMALLOC(sizeof *ctx,
                                                  NULL, DYNAMIC_TYPE_TMP_BUFFER);
 	if (ctx){
-      WOLFSSL_ENTER("wolfSSL_EVP_CIPHER_CTX_new");  
+      WOLFSSL_ENTER("wolfSSL_EVP_CIPHER_CTX_new");
 		  wolfSSL_EVP_CIPHER_CTX_init(ctx);
   }
 	return ctx;
@@ -172,15 +219,17 @@ static int evpCipherBlock(WOLFSSL_EVP_CIPHER_CTX *ctx,
                                    unsigned char *out,
                                    const unsigned char *in, int inl)
 {
+    int ret = 0;
+
     switch (ctx->cipherType) {
     #if !defined(NO_AES) && defined(HAVE_AES_CBC)
         case AES_128_CBC_TYPE:
         case AES_192_CBC_TYPE:
         case AES_256_CBC_TYPE:
             if (ctx->enc)
-                wc_AesCbcEncrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesCbcEncrypt(&ctx->cipher.aes, out, in, inl);
             else
-                wc_AesCbcDecrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesCbcDecrypt(&ctx->cipher.aes, out, in, inl);
             break;
     #endif
     #if !defined(NO_AES) && defined(WOLFSSL_AES_COUNTER)
@@ -188,9 +237,9 @@ static int evpCipherBlock(WOLFSSL_EVP_CIPHER_CTX *ctx,
         case AES_192_CTR_TYPE:
         case AES_256_CTR_TYPE:
             if (ctx->enc)
-                wc_AesCtrEncrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesCtrEncrypt(&ctx->cipher.aes, out, in, inl);
             else
-                wc_AesCtrEncrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesCtrEncrypt(&ctx->cipher.aes, out, in, inl);
             break;
     #endif
     #if !defined(NO_AES) && defined(HAVE_AES_ECB)
@@ -198,43 +247,45 @@ static int evpCipherBlock(WOLFSSL_EVP_CIPHER_CTX *ctx,
         case AES_192_ECB_TYPE:
         case AES_256_ECB_TYPE:
             if (ctx->enc)
-                wc_AesEcbEncrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesEcbEncrypt(&ctx->cipher.aes, out, in, inl);
             else
-                wc_AesEcbDecrypt(&ctx->cipher.aes, out, in, inl);
+                ret = wc_AesEcbDecrypt(&ctx->cipher.aes, out, in, inl);
             break;
     #endif
     #ifndef NO_DES3
         case DES_CBC_TYPE:
             if (ctx->enc)
-                wc_Des_CbcEncrypt(&ctx->cipher.des, out, in, inl);
+                ret = wc_Des_CbcEncrypt(&ctx->cipher.des, out, in, inl);
             else
-                wc_Des_CbcDecrypt(&ctx->cipher.des, out, in, inl);
+                ret = wc_Des_CbcDecrypt(&ctx->cipher.des, out, in, inl);
             break;
         case DES_EDE3_CBC_TYPE:
             if (ctx->enc)
-                wc_Des3_CbcEncrypt(&ctx->cipher.des3, out, in, inl);
+                ret = wc_Des3_CbcEncrypt(&ctx->cipher.des3, out, in, inl);
             else
-                wc_Des3_CbcDecrypt(&ctx->cipher.des3, out, in, inl);
+                ret = wc_Des3_CbcDecrypt(&ctx->cipher.des3, out, in, inl);
             break;
         #if defined(WOLFSSL_DES_ECB)
         case DES_ECB_TYPE:
-            wc_Des_EcbEncrypt(&ctx->cipher.des, out, in, inl);
+            ret = wc_Des_EcbEncrypt(&ctx->cipher.des, out, in, inl);
             break;
         case DES_EDE3_ECB_TYPE:
-            if (ctx->enc)
-                wc_Des3_EcbEncrypt(&ctx->cipher.des3, out, in, inl);
-            else
-                wc_Des3_EcbEncrypt(&ctx->cipher.des3, out, in, inl);
+            ret = wc_Des3_EcbEncrypt(&ctx->cipher.des3, out, in, inl);
             break;
-        #endif
-    #endif
+        #endif /* WOLFSSL_DES_ECB */
+    #endif /* !NO_DES3 */
         default:
             return 0;
-        }
-        (void)in;
-        (void)inl;
-        (void)out;
-        return 1;
+    }
+
+    if (ret != 0)
+        return 0; /* failure */
+
+    (void)in;
+    (void)inl;
+    (void)out;
+
+    return 1; /* success */
 }
 
 WOLFSSL_API int wolfSSL_EVP_CipherUpdate(WOLFSSL_EVP_CIPHER_CTX *ctx,
@@ -277,7 +328,7 @@ WOLFSSL_API int wolfSSL_EVP_CipherUpdate(WOLFSSL_EVP_CIPHER_CTX *ctx,
     blocks = inl / ctx->block_size;
     if (blocks > 0) {
         /* process blocks */
-        if (evpCipherBlock(ctx, out, ctx->buf, blocks) == 0)
+        if (evpCipherBlock(ctx, out, in, blocks*ctx->block_size) == 0)
             return 0;
         PRINT_BUF(ctx->buf, ctx->block_size);
         PRINT_BUF(out,      ctx->block_size);
@@ -327,22 +378,30 @@ WOLFSSL_API int  wolfSSL_EVP_CipherFinal(WOLFSSL_EVP_CIPHER_CTX *ctx,
                                    unsigned char *out, int *outl)
 {
     int fl ;
-    if (ctx == NULL) return BAD_FUNC_ARG;
+    if (ctx == NULL || out == NULL) return BAD_FUNC_ARG;
     WOLFSSL_ENTER("wolfSSL_EVP_CipherFinal");
     if (ctx->flags & WOLFSSL_EVP_CIPH_NO_PADDING) {
+        if (ctx->bufUsed != 0) return 0;
         *outl = 0;
         return 1;
     }
     if (ctx->enc) {
-        if (ctx->bufUsed > 0) {
+        if (ctx->block_size == 1){
+            *outl = 0; return 1;
+        }
+        if ((ctx->bufUsed >= 0) && (ctx->block_size != 1)) {
             padBlock(ctx);
             PRINT_BUF(ctx->buf, ctx->block_size);
             if (evpCipherBlock(ctx, out, ctx->buf, ctx->block_size) == 0)
                 return 0;
+
             PRINT_BUF(out, ctx->block_size);
             *outl = ctx->block_size;
         }
     } else {
+        if (ctx->block_size == 1){
+            *outl = 0; return 1;
+        }
         if (ctx->lastUsed){
             PRINT_BUF(ctx->lastBlock, ctx->block_size);
             if ((fl = checkPad(ctx, ctx->lastBlock)) >= 0) {
@@ -533,3 +592,244 @@ WOLFSSL_API int wolfSSL_EVP_add_digest(const WOLFSSL_EVP_MD *digest)
     /* nothing to do */
     return 0;
 }
+
+WOLFSSL_API int wolfSSL_EVP_add_cipher(const WOLFSSL_EVP_CIPHER *cipher)
+{
+    (void)cipher;
+    /* nothing to do */
+    return 0;
+}
+
+
+WOLFSSL_EVP_PKEY* wolfSSL_EVP_PKEY_new_mac_key(int type, ENGINE* e,
+                                          const unsigned char* key, int keylen)
+{
+    WOLFSSL_EVP_PKEY* pkey;
+
+    (void)e;
+
+    if (type != EVP_PKEY_HMAC || (key == NULL && keylen != 0))
+        return NULL;
+
+    pkey = wolfSSL_PKEY_new();
+    if (pkey != NULL) {
+        pkey->pkey.ptr = (char*)XMALLOC(keylen, NULL, DYNAMIC_TYPE_PUBLIC_KEY);
+        if (pkey->pkey.ptr == NULL && keylen > 0) {
+            wolfSSL_EVP_PKEY_free(pkey);
+            pkey = NULL;
+        }
+        else {
+            XMEMCPY(pkey->pkey.ptr, key, keylen);
+            pkey->pkey_sz = keylen;
+            pkey->type = pkey->save_type = type;
+        }
+    }
+
+    return pkey;
+}
+
+
+const unsigned char* wolfSSL_EVP_PKEY_get0_hmac(const WOLFSSL_EVP_PKEY* pkey,
+                                                size_t* len)
+{
+    if (pkey == NULL || len == NULL)
+        return NULL;
+
+    *len = (size_t)pkey->pkey_sz;
+
+    return (const unsigned char*)pkey->pkey.ptr;
+}
+
+
+int wolfSSL_EVP_DigestSignInit(WOLFSSL_EVP_MD_CTX *ctx,
+                               WOLFSSL_EVP_PKEY_CTX **pctx,
+                               const WOLFSSL_EVP_MD *type,
+                               WOLFSSL_ENGINE *e,
+                               WOLFSSL_EVP_PKEY *pkey)
+{
+    int hashType;
+    const unsigned char* key;
+    size_t keySz;
+
+    /* Unused parameters */
+    (void)pctx;
+    (void)e;
+
+    WOLFSSL_ENTER("EVP_DigestSignInit");
+
+    if (ctx == NULL || type == NULL || pkey == NULL)
+        return BAD_FUNC_ARG;
+
+#ifdef WOLFSSL_ASYNC_CRYPT
+    /* compile-time validation of ASYNC_CTX_SIZE */
+    typedef char async_test[WC_ASYNC_DEV_SIZE >= sizeof(WC_ASYNC_DEV) ?
+                                                                    1 : -1];
+    (void)sizeof(async_test);
+#endif
+
+    if (XSTRNCMP(type, "SHA256", 6) == 0) {
+         hashType = WC_SHA256;
+    }
+#ifdef WOLFSSL_SHA224
+    else if (XSTRNCMP(type, "SHA224", 6) == 0) {
+         hashType = WC_SHA224;
+    }
+#endif
+#ifdef WOLFSSL_SHA384
+    else if (XSTRNCMP(type, "SHA384", 6) == 0) {
+         hashType = WC_SHA384;
+    }
+#endif
+#ifdef WOLFSSL_SHA512
+    else if (XSTRNCMP(type, "SHA512", 6) == 0) {
+         hashType = WC_SHA512;
+    }
+#endif
+#ifndef NO_MD5
+    else if (XSTRNCMP(type, "MD5", 3) == 0) {
+        hashType = WC_MD5;
+    }
+#endif
+#ifndef NO_SHA
+    /* has to be last since would pick or 224, 256, 384, or 512 too */
+    else if (XSTRNCMP(type, "SHA", 3) == 0) {
+         hashType = WC_SHA;
+    }
+#endif /* NO_SHA */
+    else
+         return BAD_FUNC_ARG;
+
+    key = wolfSSL_EVP_PKEY_get0_hmac(pkey, &keySz);
+
+    if (wc_HmacInit(&ctx->hash.hmac, NULL, INVALID_DEVID) != 0)
+        return 0;
+
+    if (wc_HmacSetKey(&ctx->hash.hmac, hashType, key, (word32)keySz) != 0)
+        return 0;
+
+    ctx->macType = NID_hmac & 0xFF;
+
+    return 1;
+}
+
+
+int wolfSSL_EVP_DigestSignUpdate(WOLFSSL_EVP_MD_CTX *ctx,
+                                 const void *d, unsigned int cnt)
+{
+    WOLFSSL_ENTER("EVP_DigestSignFinal");
+
+    if (ctx->macType != (NID_hmac & 0xFF))
+        return 0;
+
+    if (wc_HmacUpdate(&ctx->hash.hmac, (const byte *)d, cnt) != 0)
+        return 0;
+
+    return 1;
+}
+
+
+int wolfSSL_EVP_DigestSignFinal(WOLFSSL_EVP_MD_CTX *ctx,
+                                unsigned char *sig, size_t *siglen)
+{
+    unsigned char digest[MAX_DIGEST_SIZE];
+    Hmac hmacCopy;
+    int hashLen, ret;
+
+    WOLFSSL_ENTER("EVP_DigestSignFinal");
+
+    if (ctx == NULL || siglen == NULL)
+        return 0;
+
+    if (ctx->macType != (NID_hmac & 0xFF))
+        return 0;
+
+    switch (ctx->hash.hmac.macType) {
+    #ifndef NO_MD5
+        case WC_MD5:
+            hashLen = WC_MD5_DIGEST_SIZE;
+            break;
+    #endif /* !NO_MD5 */
+
+    #ifndef NO_SHA
+        case WC_SHA:
+            hashLen = WC_SHA_DIGEST_SIZE;
+            break;
+    #endif /* !NO_SHA */
+
+    #ifdef WOLFSSL_SHA224
+        case WC_SHA224:
+            hashLen = WC_SHA224_DIGEST_SIZE;
+            break;
+    #endif /* WOLFSSL_SHA224 */
+
+    #ifndef NO_SHA256
+        case WC_SHA256:
+            hashLen = WC_SHA256_DIGEST_SIZE;
+            break;
+    #endif /* !NO_SHA256 */
+
+    #ifdef WOLFSSL_SHA512
+    #ifdef WOLFSSL_SHA384
+        case WC_SHA384:
+            hashLen = WC_SHA384_DIGEST_SIZE;
+            break;
+    #endif /* WOLFSSL_SHA384 */
+        case WC_SHA512:
+            hashLen = WC_SHA512_DIGEST_SIZE;
+            break;
+    #endif /* WOLFSSL_SHA512 */
+
+    #ifdef HAVE_BLAKE2
+        case BLAKE2B_ID:
+            hashLen = BLAKE2B_OUTBYTES;
+            break;
+    #endif /* HAVE_BLAKE2 */
+
+        default:
+            return 0;
+    }
+
+    if (sig == NULL) {
+        *siglen = hashLen;
+        return 1;
+    }
+
+    if ((int)(*siglen) > hashLen)
+        *siglen = hashLen;
+
+    XMEMCPY(&hmacCopy, &ctx->hash.hmac, sizeof(hmacCopy));
+    ret = wc_HmacFinal(&hmacCopy, digest) == 0;
+    if (ret == 1)
+        XMEMCPY(sig, digest, *siglen);
+
+    ForceZero(&hmacCopy, sizeof(hmacCopy));
+    ForceZero(digest, sizeof(digest));
+    return ret;
+}
+
+#endif /* WOLFSSL_EVP_INCLUDED */
+
+#if defined(OPENSSL_EXTRA) && !defined(NO_PWDBASED) && !defined(NO_SHA)
+WOLFSSL_API int wolfSSL_PKCS5_PBKDF2_HMAC_SHA1(const char *pass, int passlen,
+                                               const unsigned char *salt,
+                                               int saltlen, int iter,
+                                               int keylen, unsigned char *out)
+{
+    const char *nostring = "";
+    int ret = 0;
+
+    if (pass == NULL){
+        passlen = 0;
+        pass = nostring;
+    } else if (passlen == -1){
+        passlen = (int)XSTRLEN(pass);
+    }
+
+    ret = wc_PBKDF2((byte*)out, (byte*)pass, passlen, (byte*)salt, saltlen,
+                    iter, keylen, WC_SHA);
+    if (ret == 0)
+        return SSL_SUCCESS;
+    else
+        return SSL_FAILURE;
+}
+#endif /* OPENSSL_EXTRA && !NO_PWDBASED !NO_SHA*/
