@@ -1,6 +1,6 @@
 /* hash.h
  *
- * Copyright (C) 2006-2016 wolfSSL Inc.
+ * Copyright (C) 2006-2017 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -58,39 +58,39 @@ enum wc_HashType {
 
 typedef union {
     #ifndef NO_MD5
-        Md5 md5;
+        wc_Md5 md5;
     #endif
     #ifndef NO_SHA
-        Sha sha;
+        wc_Sha sha;
     #endif
     #ifdef WOLFSSL_SHA224
-        Sha224 sha224;
+        wc_Sha224 sha224;
     #endif
     #ifndef NO_SHA256
-        Sha256 sha256;
+        wc_Sha256 sha256;
     #endif
     #ifdef WOLFSSL_SHA384
-        Sha384 sha384;
+        wc_Sha384 sha384;
     #endif
     #ifdef WOLFSSL_SHA512
-        Sha512 sha512;
+        wc_Sha512 sha512;
     #endif
 } wc_HashAlg;
 
 /* Find largest possible digest size
    Note if this gets up to the size of 80 or over check smallstack build */
 #if defined(WOLFSSL_SHA512)
-    #define WC_MAX_DIGEST_SIZE SHA512_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_SHA512_DIGEST_SIZE
 #elif defined(WOLFSSL_SHA384)
-    #define WC_MAX_DIGEST_SIZE SHA384_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_SHA384_DIGEST_SIZE
 #elif !defined(NO_SHA256)
-    #define WC_MAX_DIGEST_SIZE SHA256_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_SHA256_DIGEST_SIZE
 #elif defined(WOLFSSL_SHA224)
-    #define WC_MAX_DIGEST_SIZE SHA224_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_SHA224_DIGEST_SIZE
 #elif !defined(NO_SHA)
-    #define WC_MAX_DIGEST_SIZE SHA_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_SHA_DIGEST_SIZE
 #elif !defined(NO_MD5)
-    #define WC_MAX_DIGEST_SIZE MD5_DIGEST_SIZE
+    #define WC_MAX_DIGEST_SIZE WC_MD5_DIGEST_SIZE
 #else
     #define WC_MAX_DIGEST_SIZE 64 /* default to max size of 64 */
 #endif
@@ -114,59 +114,32 @@ WOLFSSL_API int wc_HashFinal(wc_HashAlg* hash, enum wc_HashType type,
 
 #ifndef NO_MD5
 #include <wolfssl/wolfcrypt/md5.h>
-WOLFSSL_API void wc_Md5GetHash(Md5*, byte*);
-WOLFSSL_API void wc_Md5RestorePos(Md5*, Md5*);
-#if defined(WOLFSSL_TI_HASH)
-    WOLFSSL_API void wc_Md5Free(Md5*);
-#else
-    #define wc_Md5Free(d)
-#endif
+WOLFSSL_API int wc_Md5Hash(const byte* data, word32 len, byte* hash);
 #endif
 
 #ifndef NO_SHA
 #include <wolfssl/wolfcrypt/sha.h>
-WOLFSSL_API int wc_ShaGetHash(Sha*, byte*);
-WOLFSSL_API void wc_ShaRestorePos(Sha*, Sha*);
 WOLFSSL_API int wc_ShaHash(const byte*, word32, byte*);
-#if defined(WOLFSSL_TI_HASH)
-     WOLFSSL_API void wc_ShaFree(Sha*);
-#else
-    #define wc_ShaFree(d)
-#endif
 #endif
 
 #ifndef NO_SHA256
 #include <wolfssl/wolfcrypt/sha256.h>
-WOLFSSL_API int wc_Sha256GetHash(Sha256*, byte*);
-WOLFSSL_API void wc_Sha256RestorePos(Sha256*, Sha256*);
 WOLFSSL_API int wc_Sha256Hash(const byte*, word32, byte*);
-#if defined(WOLFSSL_TI_HASH)
-    WOLFSSL_API void wc_Sha256Free(Sha256*);
-#else
-    #define wc_Sha256Free(d)
-#endif
 
     #if defined(WOLFSSL_SHA224)
-        WOLFSSL_API int wc_Sha224GetHash(Sha224*, byte*);
         WOLFSSL_API int wc_Sha224Hash(const byte*, word32, byte*);
-        #define wc_Sha224Free(d)
     #endif /* defined(WOLFSSL_SHA224) */
 #endif
 
 #ifdef WOLFSSL_SHA512
 #include <wolfssl/wolfcrypt/sha512.h>
-WOLFSSL_API int wc_Sha512GetHash(Sha512*, byte*);
 WOLFSSL_API int wc_Sha512Hash(const byte*, word32, byte*);
-#define wc_Sha512Free(d)
 
     #if defined(WOLFSSL_SHA384)
-        WOLFSSL_API int wc_Sha384GetHash(Sha384*, byte*);
         WOLFSSL_API int wc_Sha384Hash(const byte*, word32, byte*);
-        #define wc_Sha384Free(d)
     #endif /* defined(WOLFSSL_SHA384) */
 #endif /* WOLFSSL_SHA512 */
 
-
 #ifdef __cplusplus
     } /* extern "C" */
 #endif
